#############################################################################################################################################################################
## sets up parameters for simulation 
# change_param_dict is a dictionary with the parameters that will be modified and their new values
proc paramSetup {change_param_dict param_type} {

switch $param_type {
  simpJJ {  ;#Simple Josephson Junction default parameters 
    set param_dict {    sd 0.020
		  sputter_time 3.0 
		  deposition_rate 0 
		  mu 0.50 
	  	Nb1_cmp_time 1.8
	  	Nb1_deposition_time 3.0 
	  	Alum_deposition_time 0.35 
  		Alum_oxidation_time 13 
	  	Nb2_deposition_time 2.0
		} 
  }
  default {
    error "parameter initiation failed\nthe default parameter set you slected does not exist"
  }
}

# update the user specified parameters
if {[info exists param_dict]} {
  foreach {par val} $change_param_dict {
    puts "$par: $val"
	  if {[dict exist $param_dict $par]} { ;# if the user has given a valid parameter
      puts "the value of $par is changed from \{[dict get $param_dict $par]\} to \{$val\}"
      dict set param_dict $par $val
	  } else {  ;# if the user has not given a valid parameter
      puts "parameter $par is not a part of this parameter set"
	  }
  }
} else {
  set param_dict $change_param_dict
}
#returns a dictionary with all parameter values
return $param_dict
}

###Parameter combinations######################################################################################################################################################
## Generates a dictionary for each possible parameter combination and stores all dictionaries in a list
#	param_dict: dictionary that holds the simulation parameters and their corresponding values
proc paramCombos {param_dict} {

# create a list of all parameters and a list of all possible value combinations
set param_list {}; set vals_lists {}
foreach {param val_list} $param_dict {
  lappend param_list $param; lappend vals_lists $val_list
}
set first 1
foreach vals $vals_lists {
  if {$first} {
	  set comb_list $vals; set first 0
 	} else {
    set new_comb_list {}
	  foreach comb_val $comb_list {
		  foreach val $vals {
			  lappend new_comb_list "${comb_val} ${val}"
		  }
	  }
	set comb_list $new_comb_list
 	}
} 
# create a dictionary with parameter names and values for each value combination and store the dictionaries in a "large list"
set large_list {}
foreach combo $comb_list {
  set sub_list {}
  foreach param $param_list val $combo {
    lappend sub_list $param
    lappend sub_list $val
  }
  lappend large_list $sub_list
 }
return $large_list
}
					
### Time Stamp ################################################################################################################################################################
## returns a time-based number used for naming structures
proc timeStamp {} {
set rand_num [expr { int(1000 * rand()) }]	;#get a random number between 0 and 999
if { $rand_num == 0 } { 				;# if the number is less than 100 we have to add 0s in front of it
  set rand_num 000
} elseif {$rand_num < 10 } {
	set rand_num "00$rand_num"
} elseif {$rand_num <100 } {
		set rand_num "0$rand_num"
}
set stamp "[clock microseconds]$rand_num"
return $stamp
}

### Hash File #################################################################################################################################################################
## Generates a md5 hash value based on the content of the input file
# inputFile: the file name/path to the file that will be hashed
proc hashFil {input_file} {
# reads the file as a string and returns a hash value 
set fil  [open $input_file r]	
set unique [md5 [read $fil]]
# returns a (hopefully) unique string
return $unique
}

### Make Executable File From Template ########################################################################################################################################
## Makes an executable file using a template file and a parameter dictionary
# template: the path to the template file
# tmp_file_name: the name that will be given to the new file
# param_dict: a dictionary with the relevant parameter names and values
proc mkFromTemplate {template tmp_file_name param_dict} {
	
set f [open $template]	;# makes a connection to the template file
set tf [open $tmp_file_name w];# makes the file that will be exicuted

while {[gets $f line] != -1} {	;# loops through each line of the file 

  if {[regexp {\$\w*} $line param]} { ;# will find any $ and the followng word and assing this value to param
    set params [regexp -all -inline -- {\$\w*} $line]

    for {set i 0} {$i < [llength $params]} {incr i} {
      set param [lindex $params $i]
      set trimed_param [string trimleft $param $]	;# will remove the $ from the extracted parrameter

      if {[dict exist $param_dict $trimed_param]} {	;# checks the dictionary has a parameter for this variable
			  set line [string map "$param {[dict get $param_dict $trimed_param]}" $line]
		  } ;# if there is no parameter for this variable, the line is printed, untouched to the temp file
    }
	}
	puts $tf $line		;# will write the line to a new file
}
close $f
close $tf
file attributes $tmp_file_name -permissions rwxrwxr--		;#make the file exicutible
}


### Directory Naming ##########################################################################################################################################################
## this proc returns a string based on its input dictionary and is used to consistantly name directories based on the input directory 
# dict_set: a dictionary with the parameters used to generate the string
proc nameDir {dict_set} {
  set dir_name "strs"
  foreach {param val} $dict_set {	
    #append dir_name "${param}_${val}__"
    append dir_name "_${val}"
  }
  set dir_name [string trimright $dir_name _]
  return $dir_name
}

### new Structure Generation Process ##############################################################################################################################################
## This proc generates a target number of structures and saves data from each structure in a database
# pro_template: the template that will be used to generate temporary process scripts 
# output_dir: the name of the directory where the generated structures will be stored
# partial_db_dir: the name of the directory where partial databases will be temporarily stored stored 
# pro_params: a dictionary with all parameter names and values that are used for structure generation
# dev_params: a dictionary with all parameter names and values that are used for device data extraction
# target: the target number of structures to be generated
# cpus: the number of CPUs that will be used by this proc
proc process {pro_template output_dir partial_db_dir pro_params dev_params {target 4} {cpus 2}} {

set brute_force 1

# geting param combos...
set pro_dict_list [paramCombos $pro_params]

puts "we have [llength $pro_dict_list] combinations"
foreach dict_set $pro_dict_list {

  ############ build structure directories ################
  set str_dir_name [nameDir $dict_set]
  set str_dir_path $output_dir/$str_dir_name
  file mkdir $str_dir_path
  
  ############## create simulation script inside structure directory #################
  set tmp_input "dict_set {$dict_set} dev_params {$dev_params} $dict_set partial_db_dir $partial_db_dir" 
  puts "\ntmp_input:\n$tmp_input\n"
  mkFromTemplate $pro_template "$str_dir_path/tmp_sim_script.tcl" $tmp_input

  ##### create simulation and organisation strings #####
  for {set cnt 0} {$cnt<$target} {incr cnt}  {
    set tmpdir [exec mktemp -p . -d]
    lappend tmpdirs $tmpdir			        ;# string with dir names that will passed to parallel execution
    lappend dest_paths $str_dir_path    ;# list with the directory paths for each parameter combination
	  lappend testChmd "$::FLOOXS $str_dir_path/tmp_sim_script.tcl $tmpdir 2> $tmpdir/tmp_error.log > $tmpdir/tmp_log.log" 			;# string with commands that will passed to parallel execution
  }
}

set batch 0; set str_cnt 0; set err_cnt 0; 

for {set i 0; set new_list ""} {$i<[llength $testChmd]} {} {
  puts "the count is $i and we should be able to run $cpus on this round"
  puts "batch: $batch"; incr batch
  for {set cpu 0} {($cpu<$cpus) && ($i<[llength $testChmd])} {incr cpu; incr i} {
    lappend small_tmpdirs [lindex $tmpdirs $i]
    lappend small_dest_paths [lindex $dest_paths $i]
    lappend small_cmd_list [lindex $testChmd $i]
    file mkdir [lindex $tmpdirs $i]
  }

  parallelSystem cmdList= $small_cmd_list	;# exicuting paralell commands

  foreach tmpdir $small_tmpdirs dest $small_dest_paths cmnd $small_cmd_list {
    if {[file isfile $tmpdir/output]} {
      puts "the structure in $tmpdir was generated sucessfully"
      incr str_cnt
      set str_name "[hashFil $tmpdir/output].str"
      file rename -force $tmpdir/output $dest/$str_name
      file delete -force $tmpdir
    } else {
      puts "It looks like str $tmpdir was NOT generated sucessfully"
      #file rename -force $tmpdir ${tmpdir}_failed
      file delete -force $tmpdir
      incr err_cnt
      if {($brute_force)&&($err_cnt<15)} {
        puts "str $tmpdir will be atempted generated again"
        lappend tmpdirs $tmpdir
        lappend dest_paths $dest
        lappend testChmd $cmnd
      }
    }
  }
  unset small_tmpdirs; unset small_dest_paths; unset small_cmd_list   
}
puts "structure generation complete!\n$str_cnt our of [expr [llength $pro_dict_list]*$target] target structures were sucessfully generated"

}
### Device Data Extraction ####################################################################################################################################################
## This proc extracts device data from structure files and saves it in a database
# dev_template: the path to the template that will be used to generate temporary device data extraction scripts
# equ_template: the path to the template that will be used to generate equation scripts
# str_dir: the name of the directory where the structures are stored
# partial_db_dir: the name of the directory where partial databases will be temporarily stored stored
# master_db_name: the name of the directory where previously extracted data is stored
# pro_params: a dictionary with all parameter names and values that are used for structure generation
# dev_params: a dictionary with all parameter names and values that are used for device data extraction
# cpus: the number of CPUs that will be used by this proc
proc dev_extr {dev_template equ_template str_dir partial_db_dir master_db_name pro_params dev_params {cpus 2}} {

################# preparing parameters #################
set pro_dict_list [paramCombos $pro_params]
set dev_dict_list [paramCombos $dev_params]

############# preparing counting parameters ############
file mkdir tmp_extr_files
set ext_cnt 0
set sim_cnt 0
set cpu_cnt 0
set cmd_str ""

#### creating and executing data extraction scripts ####
while {$ext_cnt<[llength $dev_dict_list]} {
set dev_dict [lindex $dev_dict_list $ext_cnt]
mkFromTemplate $equ_template equations$ext_cnt.tcl $dev_dict
  while {$sim_cnt<[llength $pro_dict_list]} {
    for {} {($cpu_cnt<$cpus)&&($sim_cnt<[llength $pro_dict_list])} {incr cpu_cnt; incr sim_cnt} {
      set pro_dict [lindex $pro_dict_list $sim_cnt] 
      set str_dir_name [nameDir $pro_dict]
      set str_dir_path "$str_dir/$str_dir_name"  ;# declare the structure directory for the current parameter combo
      set template_input "str_dir_path $str_dir_path partial_db_dir $partial_db_dir  master_db_name $master_db_name pro_dict {$pro_dict} dev_dict {$dev_dict}"
#puts "\n-------------------------------------------------------\ntemplate input:\n$template_input\n-------------------------------------------------------"
      mkFromTemplate $dev_template tmp_extr_files/tmp_extr_${sim_cnt}_${ext_cnt}.tcl $template_input
      lappend cmd_str "$::FLOOXS ./tmp_extr_files/tmp_extr_${sim_cnt}_${ext_cnt}.tcl $ext_cnt 2> ./tmp_extr_files/extr_${sim_cnt}_${ext_cnt}_error.log > ./tmp_extr_files/extr_${sim_cnt}_${ext_cnt}.log"
    }
    # this line is executed when either all extractions have been queued or when the queue is full  
    if {($cpu_cnt==$cpus)||(($sim_cnt==[llength $pro_dict_list])&&($ext_cnt==[expr [llength $dev_dict_list]-1]))} {
      parallelSystem cmdList= $cmd_str				;# exicuting paralell commands
      set cmd_str ""
      set cpu_cnt 0
    }  
  } 
# move on to the next extraction parrameter set
set  sim_cnt 0
incr ext_cnt   
}

####### deletng extraction logs and structures #########
#file delete -force tmp_extr_files
file delete {*}[glob equations*]
#file delete -force structures
}


### Combined Process and Device ###############################################################################################################################################
# A combination of process and device to be used on HiPerGator
proc sim_and_extr { } {
  puts "dummy proc for now"
  puts "\n STARTING SIMULATION"
  
  puts "\n STARTING EXTRACTION"
}

					
### Make Data Base ############################################################################################################################################################
## Creates a database and populates tables for device simulation and data extraction
# db_name: the name of the database that will be created
# pro_dict: a dictionary with the process parameters
# dev_dict: a dictionary with the device parameters 
proc mkdb {db_name pro_dict dev_dict} {
  
if { [file exist $db_name] } {  ;# makes sure a previous database is not overwritten 
  error "ERROR, this database already exists!!"
} else {
	#puts "creating a new database..."
  #puts "pro_dict:\n$pro_dict"
  #puts "dev_dict:\n$dev_dict"
	sqlite3 db1 $db_name         ;# creates and openes the database
	# create the table and declare the column names for extraction outputs
	db1 eval {CREATE TABLE process    (Structure text, AvgThickness real, SDThickness real, RMSThickness real);}
	db1 eval {CREATE TABLE device     (Structure text, Conductance real, Capacitance real);}
	db1 eval {CREATE TABLE param_type (Id INTEGER PRIMARY KEY, Parameter text, Type text);}
  db1 eval {INSERT INTO param_type (Parameter, Type) VALUES ('Structure', 'pro_data'), ('AvgThickness', 'pro_data'), ('SDThickness', 'pro_data'), ('RMSThickness', 'pro_data'), ('Conductance', 'dev_data'), ('Capacitance', 'dev_data');} 
  
	#puts "the complete param set is:\n$param_val_dict"
	foreach {pro_param pro_val} $pro_dict {
    db1 eval "INSERT INTO param_type (Parameter, Type) VALUES ('$pro_param', 'pro_param');"
		db1 eval "ALTER TABLE process ADD COLUMN $pro_param real;"	;# adds the variabl parameter columns
	}
	foreach {dev_param dev_val} $dev_dict {
    db1 eval "INSERT INTO param_type (Parameter, Type) VALUES ('$dev_param', 'dev_param');"
		db1 eval "ALTER TABLE device ADD COLUMN $dev_param real;"	;# can maybe be combined into one line.
	}
}  
}

### Merge Database ###########################################################################################################################################################
## Merges together the tables from two databases with the same structure
# master_db: the database that will absorb the merge database
# merge_db: the database that will be merged into the master database
# merge_table: the table that will be merged into the master database
proc merge_db {master_db merge_db merge_table} {
##puts "atampting to merge merge $merge_db into $master_db"
sqlite3 db1 $master_db
db1 eval "ATTACH DATABASE \"$merge_db\" as partial;
  BEGIN;
  INSERT INTO $merge_table SELECT * FROM partial.$merge_table;
  COMMIT;
  DETACH partial;"
puts "SUCCESS!!!"
}

### Merge Database Directory ##################################################################################################################################################
## Merges all databases in a directory into a master database
# master_fdb: the master database that will absorb all the databases from the directory
# merge_dir: the path to the directory that holds all the databases that will be merged
# merge_table: the table that will be merged into the master database
proc merge_db_dir {master_fdb merge_dir merge_table} {

# making a list of .fdb filed in the merge directory
set save_dir [pwd]    ;# save current directory
cd $merge_dir      ;# change to structure directory
set fdb_l [glob *.fdb]  ;# save a list of all .fdb files 
cd $save_dir      ;# return to the saved directory

foreach merge_fdb $fdb_l {
  puts -nonewline "mergeing $merge_fdb into $master_fdb...\t"
  merge_db $master_fdb $merge_dir/$merge_fdb $merge_table
} 
}

### Database Information ######################################################################################################################################################
## Returns information about the selected database. STILL NEEDS WORK
# db_name: name of the database to get info from, include path if the database is not in the current directory
proc db_info db_name {
  puts "db_name: $db_name"
  sqlite3 db1 $db_name

  set cnt [db1 eval {SELECT COUNT(*) FROM data}]
  puts "the total number to rows is: $cnt"

  set db_info [db1 eval {PRAGMA table_info(data);}]
  set param_l ""
  set param_p ""

  db1 close
}


### Compare Databases ##################################################################################################################################################
## Compares the table columns of two databases
# master_db: the master database that the merge db will be tested against
# comp_db: the merge db that will be tested against the master db
# comp_table: the table that will be compared
proc comp_dbs {master_db comp_db comp_table} {
  ## master db section
  sqlite3 db1 $master_db
  set m_columns [db1 eval "select * from sqlite_master where tbl_name = '$comp_table' and type = 'table' "]
  db1 close
  
  ## merge db section
  sqlite3 db1 $comp_db
  set c_columns [db1 eval "select * from sqlite_master where tbl_name = '$comp_table' and type = 'table' "]
  db1 close

  set mismatch {}
  if {$m_columns eq $c_columns} {
    return 1
  } else {
    puts "$comp_db\tdoes not match"
    return 0
  }
}

### Compare Databases Directory ##################################################################################################################################################
## Compares the table columns of two databases
# master_db: the master database that the merge db will be tested against
# comp_dir: 
# comp_table: the table that will be compared
proc rm_corrupted_dbs {master_db comp_dir comp_table} {

# making a list of .fdb filed in the merge directory
set save_dir [pwd]    ;# save current directory
cd $comp_dir      ;# change to structure directory
set fdb_l [glob *.fdb]  ;# save a list of all .fdb files 
cd $save_dir      ;# return to the saved directory

set mismatches {}
foreach comp_fdb $fdb_l {
  if { ![comp_dbs $master_db $comp_dir/$comp_fdb $comp_table]} {
    lappend mismatches "$comp_dir/$comp_fdb"
  }
} 
if {[llength $mismatches]} {
  set bad_dbs_name "corrupted_partial_dbs"
  file mkdir $bad_dbs_name
  puts "The following [llength $mismatches] partial databases were corrupted:\n$mismatches"
  puts "These databases will be moved to $bad_dbs_name"
  file rename {*}$mismatches $bad_dbs_name
} else {
  puts "No partial databases were corrupted!!"
}

}

### Database Row Count ########################################################################################################################################################
## Counts the rows in a given database of a given table and returns the count
# db_name: name/path to the database that will be opened
# table_name: name of the table whose rows will be counted
proc dbTableRows {db_name table_name} {
  #puts "Database Name: $db_name"
  #puts "Table Name: $table_name"
  sqlite3 db1 $db_name
  set cnt [db1 eval "SELECT COUNT(*) FROM $table_name"]
  db1 close

 # puts "the total number to rows is: $cnt"
  return $cnt
}
