# /*
# The Lackner avalanche model is by T. Lackner, “Avalanche Multiplication in
# Semiconductors: A Modification of Chynoweth’s Law,” Solid-State Electronics,
# vol. 34, no. 1, pp. 33–42, 1991. Lackner derived a pseudo-local ionization rate
# in the form of a modification to the Chynoweth law, assuming stationary conditions.
# The temperature-dependent factor was introduced to the original model.
#
# The ionization coefficient takes the following form:
#
# alpha(F) = gamma*a/Z * exp(-gamma*b/F), where Z =1 + gamma*bn/F *exp(-gamma*bn/F)
# + gamma*bp/F *exp(-gamma*bp/F), gamma = tanh(hbarOmega/2kTref) /tanh(hbarOmega/2kT),
#
# the a, b, and hbarOmega are model parameters whose default values are given in
# charon::Material_Properties. The default values are good in Silicon for the range
# of electric field from 1e5 V/cm to 1e6 V/cm. F is the driving field, which can be
# GradQuasiFermi, EParallelJ, EParallelJtot, EeffParallelJ, or EeffParallelJtot.
# Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively. Using positive values for n0 and p0 can improve
# convergence for problems where strong generation–recombination occurs in regions
# with small density.
# */

proc __Lackner { Qfn Qfp Elec Hole Emob Hmob lattT damp1 damp2 } {
  
  global q
  # Scale the damping parameters
  set scaled_eRefDens $damp1 ;
  set scaled_hRefDens $damp2 ;
  #
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  proc TanH {x}	{
    set tanh "((exp($x)-exp(-$x))/(exp($x)+exp(-$x)))"
    return "$tanh"
  }
  set hbarOmega_e 0.063  ;# "hbarOmega_e:[eV]");
  
  #// Holes
	set hbarOmega_h 0.063 ;#"hbarOmega_h:[eV]
  # Compute the 'gamma' parameter in the model (unitless)
    set gammae [expr [TanH ($hbarOmega_e/(2.0*$kbBoltz*300.))] / [TanH ($hbarOmega_e/(2.0*$kbBoltz*$lattT))] ]
    set gammah [expr [TanH ($hbarOmega_h/(2.0*$kbBoltz*300.))] / [TanH ($hbarOmega_h/(2.0*$kbBoltz*$lattT))] ]
    set gammae 1.0
    set gammah 1.0

  #// ----------------------------------------------------------------------
  #// --- Driving field calculation is the same for all avalanche models ---
  #// ----------------------------------------------------------------------
	set normJe   "($Emob*$Elec*grad($Qfn))"
	set normJh   "($Hmob*$Hole*grad($Qfp))"
	set normJtot "sqrt($Emob*$Elec*dot($Qfn,$Qfn))+$Hmob*$Hole*dot($Qfp,$Qfp)))"

  #set Fe "(sqrt(abs(dot($Qfn,$Qfn))) * $Elec/($Elec + $scaled_eRefDens))"
  #set Fh "(sqrt(abs(dot($Qfp,$Qfp))) * $Hole/($Hole + $scaled_hRefDens))"
	set Fe "(sqrt(abs(dot(DevPsi,DevPsi))) * $Elec/($Elec + $scaled_eRefDens))"
	set Fh "(sqrt(abs(dot(DevPsi,DevPsi))) * $Hole/($Hole + $scaled_hRefDens))"

  #// -----------------------------------------------------------------------
  #// --- Ionization coefficients calculation depends on avalanche models ---
  #// -----------------------------------------------------------------------

	set a_e 1.316e6 ;# a_e:[1/cm]
	set b_e 1.474e6 ;# b_e:[V/cm]
	set a_h 1.818e6 ;# a_h:[1/cm]
	set b_h 2.036e6 ;# b_h:[V/cm]
  set paraZ  "(1.0 + $gammae*$b_e/$Fe * exp(-$gammae*$b_e/$Fe)\
                     + $gammah*$b_h/$Fh * exp(-$gammah*$b_h/$Fh))"
    set iicoeffe "($gammae*$a_e/$paraZ * exp(-$gammae*$b_e/$Fe))"
    
  #if ( (Fh > minField) && (Fe > 0.0) )
    set iicoeffh "($gammah*$a_h/$paraZ * exp(-$gammah*$b_h/$Fh))"

  #if (normJe > 0.)  eAvaRate = iicoeffe*normJe;
  #if (normJh > 0.)  hAvaRate = iicoeffh*normJh;
  #avalanche_rate(cell,point) = (eAvaRate + hAvaRate)*scaling;
	
  # Compute avalanche generation rate
	set eAvaRate ($iicoeffe*$normJe/$eleQ)
  #sel z= $eAvaRate
  #window; plot1d
	set hAvaRate ($iicoeffh*$normJh/$eleQ)
	
	return "\"$eAvaRate\" \"$hAvaRate\""

}
  # # Get Qfn Equation
  # puts [lindex [Lackner Qfn Qfp Elec Hole Emob Hmob 300 1e15 1e15] 0]
  # # Get Qfp Equation
  # puts [lindex [Lackner Qfn Qfp Elec Hole Emob Hmob 300 1e15 1e15] 1]


