# The University of Bologna (UniBo) impact ionization model was developed for an extended
# temperature range between 25 Celcius degree and 400 Celcius degree (see also
# the UniBoNew model, a new University of Bologna Impact Ionization Model for an
# updated version of this model). It is based on impact ionization data generated
# by the Boltzmann solver HARM [1]. It covers a wide range of electric fields
# (5e4 V/cm to 6e5 V/cm) and temperatures (300 K to 700 K). It is calibrated
# against impact ionization measurements [2][3] in the whole temperature range.
#
# [1] M. C. Vecchi and M. Rudan, “Modeling Electron and Hole Transport with Full-Band
# Structure Effects by Means of the Spherical-Harmonics Expansion of the BTE,” IEEE
# Transactions on Electron Devices, vol. 45, no. 1, pp. 230–238, 1998.
# [2] S. Reggiani et al., “Electron and Hole Mobility in Silicon at Large Operating
# Temperatures—Part I: Bulk Mobility,” IEEE Transactions on Electron Devices, vol. 49,
# no. 3, pp. 490–499, 2002.s on Electron Devices, vol. 45, no. 1, pp. 230–238, 1998.
# [3] M. Valdinoci et al., “Impact-ionization in silicon at large operating temperature,” in
# International Conference on Simulation of Semiconductor Processes and Devices
# (SISPAD), Kyoto, Japan, pp. 27–30, September 1999.
#
# The ionization coefficient takes the following form:
#
# α(F, T) = F / [a(T) + b(T) * exp( d(T)/(F + c(T)) ) ], where
#
# a(T)=a0+a1*t^a2, b(T)=b0, c(T)=c0+c1*t+c2*t^2, d(T)=d0+d1*t+d2*t^2 for electrons,
# a(T)=a0+a1*t, b(T)=b0*exp(b1*t), c(T)=c0*t^c1, d(T)=d0+d1*t+d2*t^2 for holes,
# t = T/1K.
#
# a0,a1,b0,b1,c0,c1,c2,d0,d1,and d2 are model parameters whose default values are
# given in charon::Material_Properties. F is the driving field,
# i.e., GradQuasiFermi, EParallelJ, EParallelJtot, EeffParallelJ, or EeffParallelJtot.
# Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters and default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively. Using positive values for n0 and p0 can improve
# convergence for problems where strong generation–recombination occurs in regions
# with small density.

proc __UniBoHole { DrivingForce lattT Current } {
  
  set Fh $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  
  set normT [expr $lattT/1.0]
  
  # Holes
  set a0_h 2.376     ;#a0_h:[V]
  set a1_h 1.033e-2  ;#a1_h:[V]
  set a2_h 0.0       ;#a2_h:[1]
  set b0_h 0.17714   ;#b0_h:[V]
  set b1_h -2.178e-3 ;#b1_h:[1]
  set c0_h 9.47e-3   ;#c0_h:[V/cm]
  set c1_h 2.4924    ;#c1_h:[1]
  set c2_h 0.0       ;#c2_h:[1]
  set d0_h 1.4043e6  ;#d0_h:[V/cm]
  set d1_h 2.9744e3  ;#d1_h:[V/cm]
  set d2_h 1.4829    ;#d2_h:[V/cm]
  
  set ah "($a0_h + $a1_h*$normT)"
  set bh "($b0_h * exp($b1_h * $normT))"
  set ch "($c0_h * ($normT ^ $c1_h))"
  set dh "($d0_h + $d1_h*$normT + $d2_h*($normT ^ 2.0))"
  
  set iicoeffh "($Fh / ($ah + $bh * exp($dh/($Fh + $ch)) ) )"
  
  # Compute avalanche generation rate
  set normJh $Current
  set hAvaRate ($iicoeffh*$normJh/$eleQ)
  
  return $hAvaRate
  
}

proc __UniBoElec { DrivingForce lattT Current } {
  
  set Fe $DrivingForce
  
  # Retrieve physical constants
  set eleQ    1.602176634e-19 ;#       electron charge in [C]
  
  set normT [expr $lattT/1.0]
  
  # Electrons
  set a0_e 4.3383    ;#a0_e:[V]
  set a1_e -2.42e-12 ;#a1_e:[V]
  set a2_e 4.1233    ;#a2_e:[1]
  set b0_e 0.235     ;#b0_e:[V]
  set b1_e 0.0       ;#b1_e:[1]
  set c0_e 1.6831e4  ;#c0_e:[V/cm]
  set c1_e 4.3796    ;#c1_e:[V/cm]
  set c2_e 0.13005   ;#c2_e:[V/cm]
  set d0_e 1.2337e6  ;#d0_e:[V/cm]
  set d1_e 1.2039e3  ;#d1_e:[V/cm]
  set d2_e 0.56703   ;#d2_e:[V/cm]
  
  set ae "($a0_e + $a1_e*($normT ^ $a2_e))"
  set be "($b0_e)"
  set ce "($c0_e + $c1_e*$normT + $c2_e*($normT ^ 2.0))"
  set de "($d0_e + $d1_e*$normT + $d2_e*($normT ^ 2.0))"
  
  set iicoeffe "($Fe / ($ae + $be * exp($de/($Fe + $ce)) ) )"
  
  # Compute avalanche generation rate
  set normJe $Current
  set eAvaRate ($iicoeffe*$normJe/$eleQ)
  
  return $eAvaRate
  
}

# source DrivingForce.tcl
# # Get Qfn Equation
# puts [UniBoHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [UniBoElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]


