namespace eval Silicon {
# /*
# The vanOverstraeten avalanche model is R. van Overstraeten and H. de Man,
# “Measurement of the Ionization Rates in Diffused Silicon p-n Junctions,”
# Solid-State Electronics, vol. 13, no. 1, pp. 583–608, 1970.
#
# The ionization coefficient takes the following form:
#
# alpha(F) = gamma*a*exp(-gamma*b/F),
#
# where gamma = tanh(hbarOmega/2kTref)/tanh(hbarOmega/2kT),
# the a, b, and hbarOmega are model parameters whose default values are given in
# charon::Material_Properties. The low values are good in Silicon for electric field
# from 1.75e5 V/cm to E0, and the high values are valid for E0 to 6e5 V/cm. E0 is
# a critical field and its default value is 4e5 V/cm.
# F is the driving field, which can be GradQuasiFermi, EParallelJ, EParallelJtot,
# EeffParallelJ, or EeffParallelJtot. Eeff is computed by charon::FEM_ElectricField.
#
# For numerical reasons, the field F is computed as n*Fn/(n+n0) for electrons,
# and p*Fp/(p+p0) for holes. n0 and p0 are damping parameters, default to 0 and
# can be set in the Avalanche Generation section through eDrForceRefDens and
# hDrForceRefDens respectively.
# Using positive values for n0 and p0 can improve convergence for problems where
# strong generation–recombination occurs in regions with small density.
# */

proc vanOberstraetenHole args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= F=Emfp"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Driving force (F) is assumed to be the alagator field set to the gradient of the hole quasi-Fermi level called Emfp.}
  set returns {alagator string}
  set description {Avalanche breakdown for holes in silicon.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Fh [__GetValue $args F ]
  if {$Fh == ""} { set Fh Emfp }

  # Retrieve physical constants
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Holes
  set al_h        1.582e6 ;#al_h:[1/cm]
  set bl_h        2.036e6 ;#bl_h:[V/cm]
  set ah_h        6.71e5  ;#ah_h:[1/cm]
  set bh_h        1.693e6 ;#bh_h:[V/cm]
  set E0_h        4e5     ;#E0_h:[V/cm]
  set hbarOmega_h 0.063   ;#hbarOmega_h:[eV]
  
  proc TanH {x}	{
    set tanh "((exp($x)-exp(-$x))/(exp($x)+exp(-$x)))"
    return "$tanh"
  }
  
  set gamma_h    "[ expr [ TanH "($hbarOmega_h/(2.0*$kbBoltz*300.0))"] ] / [ TanH "($hbarOmega_h/(2.0*$kbBoltz*($T)))"]"
  
  set iicoeffh "(($Fh > $E0_h) ? ($gamma_h * $ah_h * exp(-$gamma_h*$bh_h/$Fh)) : ($gamma_h * $al_h * exp(-$gamma_h*$bl_h/$Fh)))"
  
  # Compute avalanche generation rate
  set hAvaRate ($iicoeffh)
  
  return $hAvaRate
}

proc vanOberstraetenElec args {
  set name [lindex [info level 0] 0]
  lappend usage "$name T= F=Emfn"
  lappend usage {Temperature (T) is in Kelvin.}
  lappend usage {Driving force (F) is assumed to be the alagator field set to the gradient of the electron quasi-Fermi level called Emfn.}
  set returns {alagator string}
  set description {Avalanche breakdown for electrons in silicon.}
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }

  set T [__GetValue $args T "$name Error: No temperature (T) specified"]
  set Fe [__GetValue $args F ]
  if {$Fe == ""} { set Fe Emfn }

  # Retrieve physical constants
  set kbBoltz 8.61733034e-5   ;#       Boltzmann constant in [eV/K]
  
  # Electrons
  set al_e        7.03e5  ;#al_e:[1/cm]
  set bl_e        1.231e6 ;#bl_e:[V/cm]
  set ah_e        7.03e5  ;#ah_e:[1/cm]
  set bh_e        1.231e6 ;#bh_e:[V/cm]
  set E0_e        4e5     ;#E0_e:[V/cm]
  set hbarOmega_e 0.063 ;#hbarOmega_e:[eV]
  
  proc TanH {x}	{
    set tanh "((exp($x)-exp(-$x))/(exp($x)+exp(-$x)))"
    return "$tanh"
  }
  
  set gamma_e    "[expr [ TanH "($hbarOmega_e/(2.0*$kbBoltz*300.0))"]] / [ TanH "($hbarOmega_e/(2.0*$kbBoltz*($T)))"]"
  
  # Compute impact ionization coefficients [1/cm]
  set iicoeffe "(($Fe > $E0_e) ? ($gamma_e * $ah_e * exp(-$gamma_e*$bh_e/$Fe) ) : ($gamma_e * $al_e * exp(-$gamma_e*$bl_e/$Fe)) )"
  
  # Compute avalanche generation rate
  set eAvaRate ($iicoeffe)
  
  return $eAvaRate
}

# source DrivingForce.tcl
# # Get Qfn Equation
# puts [vanOberstraetenHole [DrivingForceEField DevPsi Hole 1e18] 300 [Current Qfp Hmob Hole]]
# # Get Qfp Equation
# puts [vanOberstraetenElec [DrivingForceEField DevPsi Elec 1e18] 300 [Current Qfn Emob Elec]]



}
