    # Adapted from http://www.cse.unsw.edu.au/~lambert/splines/source.html
    # (c) 2002 Jonas Beskow

    #
    # Synopsis:
    #
    # spline::getSpline <vertex list> step
    #
    # Args:.  vertex list is of the form {x0 y0 x1 y1 ... xn yn} 
    #         step is the desired x-sampling step of the spline function
    #         (catmull-rom splines, passes through the vertices)
    #         
    # Result: spline points formatted as a list of the form {x0 y0 x1 y1 x2 y2}
    #         the spline will start at x1 and end at x{n-1}
    #
    # spline::getPoint vertexlistvar x
    #
    # Args: vertexlistvar is a variable containing a list of vertices.
    #       x is an x-coordinate lying between x1 and x{n-1} of the vertex list 
    #       The vertexlist will be pruned so vertices earlier in the list that
    #       those needed to calculate the point at "x" are removed.
    #       This will significantly speed up subsequent calls to getPoint for
    #       long vertex lists.
    #       
    # Result: returns the y-value of the spline function at point x.
    #         
    # Anpassung an reellen Bedarf: Nur noch Catmull-Splines, und diese werden
    # nicht entlang der X-Achse, sondern der Längenkoordinate des Splines auf-
    # gezogen. Dadurch entfällt die unpraktische Einschränkung stetig steigen-
    # der X-Koordinaten! Allerdings geht dann alles nur noch halb so schnell.
    # 
    # Peter Kämpf 2005
    #
    namespace eval spline {

        proc getPoint { vertexvar x } {
            upvar $vertexvar vertices
    #
    # puts "vertexlist length: [llength $vertices]"
    # Scan through vertex list to find four consecutive points
    # where the input x lies between the middle two
    # After the scan the list is pruned to shorten the search next time
    #
            if {$x < [lindex $vertices 2] || $x > [lindex $vertices end-3]} {
                error "x-value out of range ([lindex $vertices 2] <= x <= [lindex $vertices end-3])" 
            }
    #
            set i 0
            foreach \
              {vx(-2) vy(-2)} [lrange $vertices 0 end-6] \
              {vx(-1) vy(-1)} [lrange $vertices 2 end-4] \
              {vx(0)  vy(0)}  [lrange $vertices 4 end-2] \
              {vx(1)  vy(1)}  [lrange $vertices 6 end] {
                if {$vx(-1) <= $x && $vx(0) > $x} break
                incr i 2
            }
    #
    # Pruning
    #
            if {$i > 0} {set vertices [lrange $vertices $i end]}
    #
    # blend function for catmull-rom
    #
            set t [expr {($x-$vx(-1)) / ($vx(0)-$vx(-1))}]
            set y 0
            for {set j -2} {$j <= 1} {incr j} {
                switch -- $j {
                    -2 {
                        set b [expr {((-$t+2)*$t-1)*$t/2.0}]
                    }
                    -1 {
                        set b [expr {(((3*$t-5)*$t)*$t+2)/2.0}]
                    }
                    0  {
                        set b [expr {((-3*$t+4)*$t+1)*$t/2.0}]
                    }
                    1  {
                        set b [expr {($t-1)*$t*$t/2.0}]
                    }
                }
                set y [expr {$y + $b*$vy($j)}]
            }
            return $y
        }
    #
    # Aufbau der Kurve aus der Vertex-Liste. Trick: Um die Anfangs- und Endsteigung
    # zu erhalten, wird die Liste am Anfang und Ende um einen extrapolierten Punkt
    # bereichert.
    #
        proc getSpline { vertices step } {
            set x0 [expr {1.1 * [lindex $vertices 0] - 0.1 * [lindex $vertices 2]}]
            set y0 [expr {1.1 * [lindex $vertices 1] - 0.1 * [lindex $vertices 3]}]
            set x1 [expr {1.1 * [lindex $vertices end-1] - 0.1 * [lindex $vertices end-3]}]
            set y1 [expr {1.1 * [lindex $vertices end]   - 0.1 * [lindex $vertices end-2]}]
            set vertices [linsert $vertices 0 $x0 $y0]
            lappend vertices $x1 $y1
            set result [list ]
    #
    # Aufspalten in zwei Listen, die sich entlang der Längskoordinate orientieren
    #
            set Laenge 0.0
            set xListe [list 0.0 $x0]
            set yListe [list 0.0 $y0]
            foreach {x1 y1} [lrange $vertices 2 end] {
                set x [expr {$x1 - $x0}]
                set y [expr {$y1 - $y0}]
                set Laenge [expr {$Laenge + sqrt($x*$x + $y*$y)}]
                lappend xListe $Laenge $x1
                lappend yListe $Laenge $y1
                set x0 $x1; set y0 $y1
            }
    #
            set x0 [lindex $xListe 2]
            set x1 [lindex $xListe end-3]
            for {set x $x0} {$x < $x1} {set x [expr {$x+$step}]} {
                lappend result [getPoint xListe $x] [getPoint yListe $x]
            }
            return $result
        }
    #
    # Spline-Interpolation:
    # Aufspalten in zwei Listen, die sich entlang der Längskoordinate orientieren
    # (c) Peter Kämpf 2005
    #
    # This is what you call to get the cubic spline coordinates with free
    # sequence of X values.
    # 
    # Input:
    # vertices   A list of x y pairs of the points to be connected
    # Precision  Number of steps between two consecutive points
    # 
    # Output:
    # Returns a list of all x y pairs which describe the spline. The
    # number of value pairs = (number of points - 1) x Precision + 1
    # 
    # Needs:  CubicSpline, SolveTridiag
    # 
        proc Cubic { vertices Precision } {
            set result [list ]
            set Laenge 0.0
            set x0     [lindex $vertices 0]
            set y0     [lindex $vertices 1]
            set xListe [list 0.0 $x0]
            set yListe [list 0.0 $y0]
            foreach {x1 y1} [lrange $vertices 2 end] {
                set x [expr {$x1 - $x0}]
                set y [expr {$y1 - $y0}]
                set Laenge [expr {$Laenge + sqrt($x*$x + $y*$y)}]
                lappend xListe $Laenge $x1
                lappend yListe $Laenge $y1
                set x0 $x1; set y0 $y1
            }
    #
            set X [CubicSpline $xListe $Precision]
            set Y [CubicSpline $yListe $Precision]
            foreach {xLaenge xp} $X {yLaenge yp} $Y {
                lappend result $xp $yp
            }
            return $result
        }

    ##+##########################################################################
    #
    # CubicSpline - returns the x,y coordinates of the cubic spline using
    # XY control points.
    # vertices => {{x0 y0} {x1 y1} .... {xn yn}}
    #
    # XY points MUST BE SORTED by increasing x
    # 
    # by Keith Vetter, March 2003
    #
        proc CubicSpline { vertices {PRECISION 10} } {

            set np [expr {[llength $vertices] / 2}]
            if {$np <= 1} return

            set idx 0
            foreach {x y} $vertices {
                set X($idx) $x
                set Y($idx) $y
                incr idx
            }
            
            for {set i 1; set last $X(0)} {$i < $np} {set last $X($i); incr i} {
                set h($i) [expr {double($X($i) - $last)}]
                if {$h($i) == 0} return
            }
            
            if {$np > 2} {
                for {set i 1} {$i < $np-1} {incr i} {
                    set i2 [expr {$i + 1}]
                    set i0 [expr {$i - 1}]
                    set diag($i) [expr {($h($i) + $h($i2))/3.0}]
                    set sup($i) [expr {$h($i2) / 6.0}]
                    set sub($i) [expr {$h($i) / 6.0}]
                    set a($i) [expr {($Y($i2) - $Y($i))/$h($i2) - \
                                      ($Y($i) - $Y($i0)) / $h($i)}]
                }
                SolveTridiag sub diag sup a [expr {$np - 2}]
            }
            set a(0) [set a([expr {$np - 1}]) 0]
        
    # Now generate the point list
            set xy [list $X(0) $Y(0)]
            for {set i 1} {$i < $np} {incr i} {
                set i0 [expr {$i - 1}]
                for {set j 1} {$j <= $PRECISION} {incr j} {
                    set t1 [expr {($h($i) * $j) / $PRECISION}]
                    set t2 [expr {$h($i) - $t1}]
                    set y [expr {((-$a($i0)/6 * ($t2 + $h($i)) * $t1 + \
                                    $Y($i0))* $t2 + (-$a($i)/6 * \
                                   ($t1+$h($i)) * $t2 + $Y($i)) * $t1)/$h($i)}]
                    set t [expr {$X($i0) + $t1}]
                    lappend xy $t $y
                }
            }
            return $xy
        }

    ##+##########################################################################
    # SolveTriDiag -- solves the linear system for tridiagoal NxN matrix A
    # using Gaussian elimination (no pivoting). Since A is sparse, we pass
    # in three diagonals:
    #     sub(i)  => a(i,i-1)    diag(i) => a(i,i)    sup(i)  => a(i,i+1)
    #
    # Result is returned in b[1:n]
    #
        proc SolveTridiag {N_sub N_diag N_sup N_b n} {
            upvar 1 $N_sub sub
            upvar 1 $N_diag diag
            upvar 1 $N_sup sup
            upvar 1 $N_b b

    # Factorization and forward substitution
            for {set i 2} {$i <= $n} {incr i} {
                set i0 [expr {$i - 1}]
                set sub($i) [expr {$sub($i) / $diag($i0)}]
                set diag($i) [expr {$diag($i) - $sub($i) * $sup($i0)}]
                set b($i) [expr {$b($i) - $sub($i) * $b($i0)}]
            }
            set b($n) [expr {$b($n) / $diag($n)}]
            for {set i [expr {$n - 1}]} {$i >= 1} {incr i -1} {
                set i2 [expr {$i + 1}]
                set b($i) [expr {($b($i) - $sup($i) * $b($i2)) / $diag($i)}]
            }
        }
    }


    #------------------------------------------------------------------------------
    # test code
    # 

    proc main {} {
        global points
        global Step
        global Precision
        global cattime
        global contime
        set vertices [list 10 100  50 190 100  50 180 150 \
                          280 200 330  80 240 100 330 180 \
                          420 300 320 400 180 280]
        set Step       3
        set Precision 10
        font create big -family Helvetica -size 18 -weight bold
    #
    # Open window with canvas on the left
    #
        wm title . "Splinetest"
        pack [frame .ctrl -relief ridge -bd 2 -padx 5 -pady 5] \
            -side right -fill both -ipady 5
        pack [frame .screen -bd 2 -relief raised] -side top -fill both -expand 1
    #
        canvas .c -relief raised -borderwidth 0 -height 300 -width 500
        pack   .c -in .screen -side top -fill both -expand 1
        bind all <Alt-c> {console show}
    #
    # and control widgets on the right side
    #
        label .cat -text "Catmull-Rom Splines" -fg blue -font big
        scale .cats -orient h -variable Step -command {updateSpline .c} \
            -label Stepsize: -relief ridge -from 1 -to 50
        label .catt -text "Time to draw"
        entry .cate -textvariable cattime -width 24 -bd 0 -justify right
    #
        label .con -text "Cubic Splines" -fg green -font big
        scale .cons -orient h -variable Precision -command {updateSpline .c} \
            -label Precision: -relief ridge -from 1 -to 20
        label .cont -text "Time to draw"
        entry .cone -textvariable contime -width 24 -bd 0 -justify right
    #
        message .m -width 200 \
            -text "Click and drag to move points;\
                   Double-click to add a point;\
                   Shift-Double-click to remove a point."
        button .about -text About -command About -width 12
    #
    # Plot both lines
    #
        set points ""
        set contime [time { .c create line [spline::Cubic $vertices $Precision] \
            -width 2 -fill green -tags [list spline cube]}]
        set cattime [time { .c create line [spline::getSpline $vertices $Step] \
            -width 2 -fill blue -tags [list spline cmr]}]
        .c create line $vertices -smooth 1 -width 2 -fill orange -tags [list spline bezier]
        foreach {x y} $vertices {
            addPt .c $x $y
        }
        bind .c <Double-ButtonPress-1> [namespace code [list addPt .c %x %y]]
        bind .c <Shift-Double-ButtonPress-1> [namespace code [list rmPt .c %x %y]]
    #
    # Position the widgets
    #
        grid .cat   -in .ctrl -row 1 -sticky ew -pady 10
        grid .cats  -in .ctrl -row 2 -sticky ew -pady 10
        grid .catt  -in .ctrl -row 3 -sticky ew
        grid .cate  -in .ctrl -row 4 -sticky ew -padx 10
    #
        grid .con   -in .ctrl -row 6 -sticky ew -pady 10
        grid .cons  -in .ctrl -row 7 -sticky ew -pady 10
        grid .cont  -in .ctrl -row 8 -sticky ew
        grid .cone  -in .ctrl -row 9 -sticky ew -padx 10
    #
        grid .m     -in .ctrl -row 11 -sticky ew -pady 30
        grid rowconfigure .ctrl {1 2 3 4 6 7 8 9 11 13} -weight 1
        grid rowconfigure .ctrl {0 5 10 12}  -weight 1 -minsize 10
    #
        grid .about -in .ctrl -row 13 -pady 10
        update
    }

    proc movePt { c id x y } {
        $c coords $id $x $y $x $y
        updateSpline $c
    }

    proc addPt { c x y } {
        global points
        set id [.c create oval $x $y $x $y -width 6 -outline red]
        .c bind $id <B1-Motion> [namespace code [list movePt .c $id %x %y]]
        lappend points $id
        updateSpline $c
    }

    proc rmPt { c x y } {
        global points
        set Index 0
    #    puts "Punkt an $x $y entfernen"
        foreach id $points {
            foreach {xp yp xp yp} [$c coords $id] break
            if {[expr {abs($x - $xp)}] < 3 &&  [expr {abs($y - $yp)}] < 3} {
                set points [lreplace $points $Index $Index]
                $c delete $id
                break
            }
            incr Index
        }
        updateSpline $c
    }

    proc updateSpline { c {a 0} } {
        global points
        global Step
        global Precision
        global cattime
        global contime
    #
        foreach id $points {
            foreach {x y x y} [$c coords $id] break
            lappend vertices $x $y
        }
        if {[llength $vertices] < 4} return
        set contime [time { $c coords cube [spline::Cubic $vertices $Precision] }]
        set cattime [time { $c coords cmr  [spline::getSpline $vertices $Step] }]
        .c coords bezier $vertices
    }

    proc About {} {
        set msg "Cubic Splines by Keith Vetter, March 2003 \
            \nCatmull-Rom Splines by Jonas Beskow, 2002 \
            \nFree order of X points by Peter Kämpf, 2005"
        tk_messageBox -title About -message $msg
    }

    #
    main

