# All the following procedures are designed to directly support stress model computation in floods
# The procedures take single or multidimensional lists as arguments and return lists or values
# A value is a single scalar value, a 1D list is a vector, a 2D list is a matrix, and
# anything greater than 2D is a Tensor

proc __BlankMatrix { N } {
  set ret {}
  for { set i 0 } { $i < $N } { incr i } {
    set tmp {}
    for { set j 0 } { $j < $N } { incr j } {
      lappend tmp {}
    }
    lappend ret $tmp
  }
  return $ret
}

# Matrix A times vector x multiplication
# Returns a vector
proc __MultAx { A x } {
  if { [llength $A] != [llength $x] || [llength [lindex $A 0]] != [llength $x] } {
    puts "MultAX: ill formed matrix and vector"
    return
  }
  set ret {}
  for { set i 0 } { $i < [llength $A] } { incr i } {
    lappend ret {}
    lset ret $i [ expr [ lindex $A $i 0 ] * [ lindex $x 0 ] ]
    for { set j 1 } { $j < [llength $A] } { incr j } {
      lset ret $i [ expr 1.0*[lindex $ret $i] + 1.0*[ lindex $A $i $j ] * [ lindex $x $j ] ]
    }
  }
  return $ret
}

# Returns the transpose of matrix A
proc __Transpose { A } {
  if { [llength $A] != [llength [lindex $A 0]] } {
    puts "MultAX: ill formed matrix and vector"
    return
  }
  set N [llength $A]
  set ret [__BlankMatrix $N]
  for { set i 0 } { $i < [llength $A] } { incr i } {
    for { set j 0 } { $j < [llength $A] } { incr j } {
      lset ret $i $j [ expr [ lindex $A $j $i ] ]
    }
  }
  return $ret
}
# Takes 4 scalars and returns a 2x2 matrix
# [a b]
# [c d]
proc __Make2x2 { a b c d } {
  set ret {{{} {}} {{} {}}}
  lset ret 0 0 $a
  lset ret 0 1 $b
  lset ret 1 0 $c
  lset ret 1 1 $d
  return $ret
}

# returns the determinate of a 2x2 matrix
proc __Det2x2 { A } {
  if { [llength $A] != 2 || [llength [lindex $A 0]] != 2 || [llength [lindex $A 1]] != 2 } {
    puts "MultAx: ill formed matrix and vector"
    return
  }
  return [expr [lindex $A 0 0]*[lindex $A 1 1] - [lindex $A 0 1]*[lindex $A 1 0]]
}
# returns the inverse of a 2x2 matrix
proc __Inv2x2 { A } {
  if { [llength $A] != 2 || [llength [lindex $A 0]] != 2 || [llength [lindex $A 1]] != 2 } {
    puts "Inv2x2: ill formed matrix and vector"
    return
  }
  set det [__Det2x2 $A]
  if { $det == 0 } {
    puts "Inv2x2: Matrix is singular"
    return
  }
  return [__Make2x2 [expr  1.0*[lindex $A 1 1]/$det] [expr -1.0*[lindex $A 0 1]/$det]\
      [expr -1.0*[lindex $A 1 0]/$det] [expr  1.0*[lindex $A 0 0]/$det]]
}
#puts [__Inv2x2 {{1 1} {1 -1}}]
# laplace formula for determinate
# returns the determinate of an NxN matrix
# A is a matrix and N is the length of the sides
proc __DetNxN { A N } {
  if { [llength $A] != $N} {
    puts "DetNxN: ill formed matrix"
    return
  }
  for { set i 0 } { $i < $N } { incr i } {
    if { [llength [lindex $A $i ]] != $N } {
      puts "InvNxN: ill formed matrix"
      return
    }
  }
  set det 0
  for { set i 0 } { $i < $N } { incr i } {
    set B {}
    set l 0
    for { set j 0 } { $j < $N } { incr j } {
      if { $j == $i } { continue }
      set m 0
      for { set k 1 } { $k < $N } { incr k } {
        lset B $m $l [lindex $A $k $j]
        incr m
      }
      incr l
    }
    if { $N == 3 } {
      set det [expr $det + (-1)**$i * 1.0*[lindex $A 0 $i] * [__Det2x2 $B] ]
    } else {
      set det [expr $det + (-1)**$i * 1.0*[lindex $A 0 $i] * [__DetNxN $B [expr $N-1] ] ]
    }
  }
  return $det
}
# puts [__DetNxN {{10 1 2 4 3}
# 			  {3 4 5 7 9}
# 			  {6 7 -8 10 13}
# 	          {3 6 8 12 20}
# 	          {1 2 3 4 5 }} 5]

# multiplies two NxN matricies
proc __MultNxN { A B N } {
  if { [llength $A] != $N || [llength $B] != $N} {
    puts "MultNxN: ill formed matrix"
    return
  }
  for { set i 0 } { $i < $N } { incr i } {
    if { [llength [lindex $A $i ]] != $N || [llength [lindex $B $i ]] != $N } {
      puts "InvNxN: ill formed matrix"
      return
    }
  }
  set ret [__BlankMatrix $N]
  for { set i 0 } { $i < $N } { incr i } {
    for { set j 0 } { $j < $N } { incr j } {
      set val 0
      for { set k 0 } { $k < $N } { incr k } {
        set val [expr $val + 1.0*[lindex $A $i $k]*[lindex $B $k $j]]
      }
      lset ret $i $j $val
    }
  }
  return $ret
}
#puts [MultNxN {{1 2} {3 4}} {{1 2} {4 3}} 2]
# Returns the inverse of a 3x3 matrix
proc __Inv3x3 { A } {
  if { [llength $A] != 3} {
    puts "__Inv3x3: ill formed matrix "
    return
  }
  for { set i 0 } { $i < 3 } { incr i } {
    if { [llength [lindex $A $i ]] != 3 } {
      puts "__Inv3x3: ill formed matrix"
      return
    }
  }
  
  set det [__DetNxN $A 3]
  if { $det == 0 } {
    puts "__Inv3x3: Matrix is singular"
    return
  }
  set ret [__BlankMatrix [llength $A]]
  lset ret 0 0 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 1 1] [lindex $A 1 2]\
      [lindex $A 2 1] [lindex $A 2 2]]]/$det]
  lset ret 0 1 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 2] [lindex $A 0 1]\
      [lindex $A 2 2] [lindex $A 2 1]]]/$det]
  lset ret 0 2 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 1] [lindex $A 0 2]\
      [lindex $A 1 1] [lindex $A 1 2]]]/$det]
  
  lset ret 1 0 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 1 2] [lindex $A 1 0]\
      [lindex $A 2 2] [lindex $A 2 0]]]/$det]
  lset ret 1 1 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 0] [lindex $A 0 2]\
      [lindex $A 2 0] [lindex $A 2 2]]]/$det]
  lset ret 1 2 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 2] [lindex $A 0 0]\
      [lindex $A 1 2] [lindex $A 1 0]]]/$det]
  
  lset ret 2 0 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 1 0] [lindex $A 1 1]\
      [lindex $A 2 0] [lindex $A 2 1]]]/$det]
  lset ret 2 1 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 1] [lindex $A 0 0]\
      [lindex $A 2 1] [lindex $A 2 0]]]/$det]
  lset ret 2 2 [expr 1.0*[__Det2x2 [__Make2x2 [lindex $A 0 0] [lindex $A 0 1]\
      [lindex $A 1 0] [lindex $A 1 1]]]/$det]
  return $ret
}
#puts [__Inv3x3 {{1 1 2} {1 -1 2} {0 4 5}}]

proc 2DTensorListToFlooxsTensor args {
  lappend usage "2DTensorListToFlooxsTensor TensorList"
  set returns "Flooxs tensor alagator expression"
  set description "inputs a 2x2 matrix Tensor/ Matrix list (3 dimensional list where the first two dimensions are the 2D matrix and the 3rd dimension are other matricies). All matricies are multiplied together by component."
  if { [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  
  set List [lindex $args 0]
  
  set Ten {{{} {}} {{} {}}}
  
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 1 "([lindex $List $i 0 1])"
      lset Ten 1 0 "([lindex $List $i 1 0])"
      lset Ten 1 1 "([lindex $List $i 1 1])"
    } else {
      lset Ten 0 0 "[lindex $Ten 0 0]*([lindex $List $i 0 0])"
      lset Ten 0 1 "[lindex $Ten 0 1]*([lindex $List $i 0 1])"
      lset Ten 1 0 "[lindex $Ten 1 0]*([lindex $List $i 1 0])"
      lset Ten 1 1 "[lindex $Ten 1 1]*([lindex $List $i 1 1])"
    }
  }
  
	set out "Tensor([lindex $Ten 0 0], [lindex $Ten 0 1],\
      [lindex $Ten 1 0], [lindex $Ten 1 1])"
  
  return $out
}

proc TensorAdd args {
  lappend usage "TensorAdd TensorList"
  set returns "TensorList"
  set description "Takes a tensor list and adds them together returning a tensor list of length one"
  if { [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  
  set List [lindex $args 0]
  
  set Ten {{{} {} {}} {{} {} {}} {{} {} {}}}
  
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 1 "([lindex $List $i 0 1])"
      lset Ten 0 2 "([lindex $List $i 0 2])"
      lset Ten 1 0 "([lindex $List $i 1 0])"
      lset Ten 1 1 "([lindex $List $i 1 1])"
      lset Ten 1 2 "([lindex $List $i 1 2])"
      lset Ten 2 0 "([lindex $List $i 2 0])"
      lset Ten 2 1 "([lindex $List $i 2 1])"
      lset Ten 2 2 "([lindex $List $i 2 2])"
    } else {
      lset Ten 0 0 "[lindex $Ten 0 0]+([lindex $List $i 0 0])"
      lset Ten 0 1 "[lindex $Ten 0 1]+([lindex $List $i 0 1])"
      lset Ten 0 2 "[lindex $Ten 0 2]+([lindex $List $i 0 2])"
      lset Ten 1 0 "[lindex $Ten 1 0]+([lindex $List $i 1 0])"
      lset Ten 1 1 "[lindex $Ten 1 1]+([lindex $List $i 1 1])"
      lset Ten 1 2 "[lindex $Ten 1 2]+([lindex $List $i 1 2])"
      lset Ten 2 0 "[lindex $Ten 2 0]+([lindex $List $i 2 0])"
      lset Ten 2 1 "[lindex $Ten 2 1]+([lindex $List $i 2 1])"
      lset Ten 2 2 "[lindex $Ten 2 2]+([lindex $List $i 2 2])"
    }
  }
  
  return $Ten
}

proc TensorMult args {
  lappend usage "TensorMult TensorList"
  set returns "TensorList"
  set description "Takes a tensor list and multiplies them together returning a tensor list of length one"
  if { [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  set List [lindex $args 0]
  
  set Ten {{{} {} {}} {{} {} {}} {{} {} {}}}
  
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 1 "([lindex $List $i 0 1])"
      lset Ten 0 2 "([lindex $List $i 0 2])"
      lset Ten 1 0 "([lindex $List $i 1 0])"
      lset Ten 1 1 "([lindex $List $i 1 1])"
      lset Ten 1 2 "([lindex $List $i 1 2])"
      lset Ten 2 0 "([lindex $List $i 2 0])"
      lset Ten 2 1 "([lindex $List $i 2 1])"
      lset Ten 2 2 "([lindex $List $i 2 2])"
    } else {
      lset Ten 0 0 "[lindex $Ten 0 0]*([lindex $List $i 0 0])"
      lset Ten 0 1 "[lindex $Ten 0 1]*([lindex $List $i 0 1])"
      lset Ten 0 2 "[lindex $Ten 0 2]*([lindex $List $i 0 2])"
      lset Ten 1 0 "[lindex $Ten 1 0]*([lindex $List $i 1 0])"
      lset Ten 1 1 "[lindex $Ten 1 1]*([lindex $List $i 1 1])"
      lset Ten 1 2 "[lindex $Ten 1 2]*([lindex $List $i 1 2])"
      lset Ten 2 0 "[lindex $Ten 2 0]*([lindex $List $i 2 0])"
      lset Ten 2 1 "[lindex $Ten 2 1]*([lindex $List $i 2 1])"
      lset Ten 2 2 "[lindex $Ten 2 2]*([lindex $List $i 2 2])"
    }
  }
  
  return $Ten
}

# Input: 3x3 Tensor/ Matrix list (3 dimensional list where the first two dimensions
# are the 2D matrix and the 3rd dimension are other matricies)
# All matricies are added together by component
# Returns: Flooxs tensor alagator expression
proc 3DTensorListToFlooxsTensorAdd args {
  lappend usage "3DTensorListToFlooxsTensorAdd TensorList"
  set returns "Flooxs tensor alagator expression"
  set description "inputs a 3x3 matrix Tensor/ Matrix list (3 dimensional list where the first two dimensions are the 3D matrix and the 3rd dimension are other matricies). All matricies are added together by component."
  if { [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  
  set List [lindex $args 0]
  
  set Ten [TensorAdd $List]
  
	set out "Tensor([lindex $Ten 0 0], [lindex $Ten 0 1], [lindex $Ten 0 2],\
					[lindex $Ten 1 0], [lindex $Ten 1 1], [lindex $Ten 1 2],\
      [lindex $Ten 2 0], [lindex $Ten 2 1], [lindex $Ten 2 2])"
  
  return $out
}


# Input: 3x3 Tensor/ Matrix list (3 dimensional list where the first two dimensions
# are the 2D matrix and the 3rd dimension are other matricies)
# All matricies are multiplied together by component
# Returns: Flooxs tensor alagator expression
proc 3DTensorListToFlooxsTensor args {
  lappend usage "3DTensorListToFlooxsTensor TensorList"
  set returns "Flooxs tensor alagator expression"
  set description "inputs a 3x3 matrix Tensor/ Matrix list (3 dimensional list where the first two dimensions are the 3D matrix and the 3rd dimension are other matricies). All matricies are multiplied together by component."
  if { [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  
  set List [lindex $args 0]
  
  set Ten {{{} {} {}} {{} {} {}} {{} {} {}}}
  
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 1 "([lindex $List $i 0 1])"
      lset Ten 0 2 "([lindex $List $i 0 2])"
      lset Ten 1 0 "([lindex $List $i 1 0])"
      lset Ten 1 1 "([lindex $List $i 1 1])"
      lset Ten 1 2 "([lindex $List $i 1 2])"
      lset Ten 2 0 "([lindex $List $i 2 0])"
      lset Ten 2 1 "([lindex $List $i 2 1])"
      lset Ten 2 2 "([lindex $List $i 2 2])"
    } else {
      lset Ten 0 0 "[lindex $Ten 0 0]*([lindex $List $i 0 0])"
      lset Ten 0 1 "[lindex $Ten 0 1]*([lindex $List $i 0 1])"
      lset Ten 0 2 "[lindex $Ten 0 2]*([lindex $List $i 0 2])"
      lset Ten 1 0 "[lindex $Ten 1 0]*([lindex $List $i 1 0])"
      lset Ten 1 1 "[lindex $Ten 1 1]*([lindex $List $i 1 1])"
      lset Ten 1 2 "[lindex $Ten 1 2]*([lindex $List $i 1 2])"
      lset Ten 2 0 "[lindex $Ten 2 0]*([lindex $List $i 2 0])"
      lset Ten 2 1 "[lindex $Ten 2 1]*([lindex $List $i 2 1])"
      lset Ten 2 2 "[lindex $Ten 2 2]*([lindex $List $i 2 2])"
    }
  }
  
	set out "Tensor([lindex $Ten 0 0], [lindex $Ten 0 1], [lindex $Ten 0 2],\
					[lindex $Ten 1 0], [lindex $Ten 1 1], [lindex $Ten 1 2],\
      [lindex $Ten 2 0], [lindex $Ten 2 1], [lindex $Ten 2 2])"
  
  return $out
}

# Convert a 3D tensor to 2D by chopping off the z component
proc 3DTensorTo2D args {
  lappend usage "3DTensorTo2D TensorList"
  set returns "TensorList"
  set description "Convert a 3D tensor to 2D by chopping off the z component."
  if { [llength $args] != 1 || [__Help $args [lindex [info level 0] 0] $usage $returns $description] } {
    return ""
  }
  set Ten [lindex $args 0]
  
  set tmp {{{{} {}} {{} {}}}}
  lset tmp 0 0 0 [lindex $Ten 0 0 0]
  lset tmp 0 0 1 [lindex $Ten 0 0 1]
  lset tmp 0 1 0 [lindex $Ten 0 1 0]
  lset tmp 0 1 1 [lindex $Ten 0 1 1]
  return $tmp
}

# takes a carrier vector
# returns a tensor 3x3 list
proc GetAnisoCarrierTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name CarrierVector"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Carrier [lindex $args 0]
  set Carrier_X [lindex $Elecs 0 ]
  set Carrier_Y [lindex $Elecs 1 ]
  set Carrier_Z [lindex $Elecs 2 ]
  set T11      "($Carrier_X)"
  set T12      "(0)"
  set T13      "(0)"
  set T21      "(0)"
  set T22      "($Carrier_Y)"
  set T23      "(0)"
  set T31      "(0)"
  set T32      "(0)"
  set T33      "($Carrier_Z)"
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset Ten 0 0 0 "$T11"
  lset Ten 0 0 1 "$T12"
  lset Ten 0 0 2 "$T13"
  lset Ten 0 1 0 "$T21"
  lset Ten 0 1 1 "$T22"
  lset Ten 0 1 2 "$T23"
  lset Ten 0 2 0 "$T31"
  lset Ten 0 2 1 "$T32"
  lset Ten 0 2 2 "$T33"
  return $Ten
}
proc GetCarrierTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Carrier"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Carrier [lindex $args 0]
  set T11      "($Carrier)"
  set T12      "($Carrier)"
  set T13      "($Carrier)"
  set T21      "($Carrier)"
  set T22      "($Carrier)"
  set T23      "($Carrier)"
  set T31      "($Carrier)"
  set T32      "($Carrier)"
  set T33      "($Carrier)"
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset Ten 0 0 0 "$T11"
  lset Ten 0 0 1 "$T12"
  lset Ten 0 0 2 "$T13"
  lset Ten 0 1 0 "$T21"
  lset Ten 0 1 1 "$T22"
  lset Ten 0 1 2 "$T23"
  lset Ten 0 2 0 "$T31"
  lset Ten 0 2 1 "$T32"
  lset Ten 0 2 2 "$T33"
  return $Ten
}

# takes a mobility vector
# returns a tensor 3x3 list
proc GetMobilityTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name MobilityVector"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Mobs [lindex $args 0]
  set Mob1 [lindex $Mobs 0]
  set Mob2 [lindex $Mobs 1]
  set Mob3 [lindex $Mobs 2]
  set Mob4 [lindex $Mobs 3]
  if { $Mob4 == "" } { set Mob4 0 }
  set Mob5 [lindex $Mobs 4]
  if { $Mob5 == "" } { set Mob5 0 }
  set Mob6 [lindex $Mobs 5]
  if { $Mob6 == "" } { set Mob6 0 }
  #set Mob4 0.
  #set Mob5 0.
  #set Mob6 0.
  
  set T11      "($Mob1)"
  set T12      "($Mob6)"
  set T13      "($Mob5)"
  set T21      "($Mob6)"
  set T22      "($Mob2)"
  set T23      "($Mob4)"
  set T31      "($Mob5)"
  set T32      "($Mob4)"
  set T33      "($Mob3)"
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset Ten 0 0 0 "$T11"
  lset Ten 0 0 1 "$T12"
  lset Ten 0 0 2 "$T13"
  lset Ten 0 1 0 "$T21"
  lset Ten 0 1 1 "$T22"
  lset Ten 0 1 2 "$T23"
  lset Ten 0 2 0 "$T31"
  lset Ten 0 2 1 "$T32"
  lset Ten 0 2 2 "$T33"
  return $Ten
}

# takes a mobility vector, carrier type
# returns a tensor 3x3 list
proc GetConductivityTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Mobilities Carriers"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set Mobs [lindex $args 0]
  set Carrier [lindex $args 1]
  set Mob1 [lindex $Mobs 0]
  set Mob2 [lindex $Mobs 1]
  set Mob3 [lindex $Mobs 2]
  set Mob4 [lindex $Mobs 3]
  set Mob5 [lindex $Mobs 4]
  set Mob6 [lindex $Mobs 5]
  
  set T11      "($Mob1)*($Carrier)"
  set T12      "($Mob6)*($Carrier)"
  set T13      "($Mob5)*($Carrier)"
  set T21      "($Mob6)*($Carrier)"
  set T22      "($Mob2)*($Carrier)"
  set T23      "($Mob4)*($Carrier)"
  set T31      "($Mob5)*($Carrier)"
  set T32      "($Mob4)*($Carrier)"
  set T33      "($Mob3)*($Carrier)"
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset Ten 0 0 0 "$T11"
  lset Ten 0 0 1 "$T12"
  lset Ten 0 0 2 "$T13"
  lset Ten 0 1 0 "$T21"
  lset Ten 0 1 1 "$T22"
  lset Ten 0 1 2 "$T23"
  lset Ten 0 2 0 "$T31"
  lset Ten 0 2 1 "$T32"
  lset Ten 0 2 2 "$T33"
  return $Ten
}

proc GetHallTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Mobilities RawMobilities Carriers HallMobility MagneticField"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args Mobs RawMobs Carrier Mob B
  set Mob1 [lindex $Mobs 0]
  set Mob2 [lindex $Mobs 1]
  set Mob3 [lindex $Mobs 2]
  set RawMob1 [lindex $RawMobs 0]
  set RawMob2 [lindex $RawMobs 1]
  set RawMob3 [lindex $RawMobs 2]
  set scal [expr 1.0/100**2 ]
  set Bx [lindex $B 0]; set By [lindex $B 1]; set Bz [lindex $B 2];
  
  set T11      " 0. "
  set T22      " 0. "
  set T33      " 0. "
  # if { [expr [lindex $Bz]] == 0. } {
  #   set T12      "(0)"
  #   set T21      "(0)"
  # } else {
    set T12      "(  ($Mob1) * ($Mob2)  * ($Carrier) * ($Mob)*$Bz*$scal)"
    set T21      "( -($Mob2) * ($Mob1)  * ($Carrier) * ($Mob)*$Bz*$scal)"
  # }
  # if { [expr [lindex $By]] == 0. } {
  #   set T13      "(0)"
  #   set T31      "(0)"
  # } else {
    set T13      "(  ($Mob1) * ($Mob3)  * ($Carrier) * ($Mob)*$By*$scal)"
    set T31      "( -($Mob3) * ($Mob1)  * ($Carrier) * ($Mob)*$By*$scal)"
  # }
  # if { [expr [lindex $Bx]] == 0. } {
  #   set T23      "(0)"
  #   set T32      "(0)"
  # } else {
    set T23      "( (($Mob2) * ($Mob3)) * ($Carrier) * ($Mob)*$Bx*$scal)"
    set T32      "(-(($Mob3) * ($Mob2)) * ($Carrier) * ($Mob)*$Bx*$scal)"
  # }
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset Ten 0 0 0 "$T11"
  lset Ten 0 0 1 "$T12"
  lset Ten 0 0 2 "$T13"
  lset Ten 0 1 0 "$T21"
  lset Ten 0 1 1 "$T22"
  lset Ten 0 1 2 "$T23"
  lset Ten 0 2 0 "$T31"
  lset Ten 0 2 1 "$T32"
  lset Ten 0 2 2 "$T33"
  #puts $Ten
  #gets stdin
  return $Ten
}

# Takes a 3 length list and crystal rotation vector
proc __RotateVector3 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Vector miller"
  set returns "3 long list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args x miller
  set A [ __MillerToRotation $miller ]
  set ret {}
  for { set i 0 } { $i < [llength $A] } { incr i } {
    lappend ret {}
    for { set j 0 } { $j < [llength $A] } { incr j } {
      if { abs([ lindex $A $i $j ]) > 1e-10 }  {
        if { [lindex $ret $i] == "" } {
          lset ret $i "([expr [ lindex $A $i $j ]**2]*[ lindex $x $j ])"
        } else {
          lset ret $i "([lindex $ret $i]+[expr [ lindex $A $i $j ]**2]*[ lindex $x $j ])"
        }
      }
    }
  }
  return $ret
}

# Takes a 3x3 Tensor list and crystal rotation vector
# returns a rotated tensor
proc __RotateTensor args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Tensor miller"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args Tensor miller
  set TensorVec "0.0 0.0 0.0 0.0 0.0 0.0"
  lset TensorVec 0 [lindex $Tensor 0 0 0]
  lset TensorVec 1 [lindex $Tensor 0 1 1]
  lset TensorVec 2 [lindex $Tensor 0 2 2]
  lset TensorVec 3 [lindex $Tensor 0 1 2]
  lset TensorVec 4 [lindex $Tensor 0 0 2]
  lset TensorVec 5 [lindex $Tensor 0 0 1]
  set alpha [__approxFirstToSecondOrderRotation [ __MillerToRotation $miller ]]
  set newTensorVec ""
  for { set i 0 } { $i < 6 } { incr i } {
    lappend newTensorVec ""
    for { set j 0 } { $j < 6 } { incr j } {
      if {[lindex $alpha $i $j] == "-0.0"  || [lindex [split [lindex $TensorVec $j] "*"] 0] == "(-0.0)"} { continue; }
      if {[lindex $alpha $i $j] == "0.0"  || [lindex [split [lindex $TensorVec $j] "*"] 0] == "(0.0)"} { continue; }
      if { [lindex $newTensorVec $i] == "" } {
        lset newTensorVec $i "[expr abs([lindex $alpha $i $j])]*([lindex $TensorVec $j])"
      } else {
        lset newTensorVec $i "[lindex $newTensorVec $i]+[expr abs([lindex $alpha $i $j])]*([lindex $TensorVec $j])"
      }
    }
  }
  #puts $newTensorVec
  #gets stdin
  for { set i 0 } { $i < 6 } { incr i } {
    if {[lindex $newTensorVec $i] == ""} {
      lset newTensorVec $i "0.0"
    }
  }
  set newTensor {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  lset newTensor 0 0 0 [lindex $newTensorVec 0]
  lset newTensor 0 0 1 [lindex $newTensorVec 5]
  lset newTensor 0 0 2 [lindex $newTensorVec 4]
  lset newTensor 0 1 0 [lindex $newTensorVec 5]
  lset newTensor 0 1 1 [lindex $newTensorVec 1]
  lset newTensor 0 1 2 [lindex $newTensorVec 3]
  lset newTensor 0 2 0 [lindex $newTensorVec 4]
  lset newTensor 0 2 1 [lindex $newTensorVec 3]
  lset newTensor 0 2 2 [lindex $newTensorVec 2]
  lset newTensor 0 0 0 [lindex $newTensorVec 0]
  return $newTensor
}
# Same procedure but only outputs Diag
proc __RotateTensorDiag args {
  set name [lindex [info level 0] 0]
  lappend usage "$name Tensor miller"
  set returns "tensor 3x3 list"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args Tensor miller
  set TensorVec "0.0 0.0 0.0 0.0 0.0 0.0"
  lset TensorVec 0 [lindex $Tensor 0 0 0]
  lset TensorVec 1 [lindex $Tensor 0 1 1]
  lset TensorVec 2 [lindex $Tensor 0 2 2]
  lset TensorVec 3 [lindex $Tensor 0 1 2]
  lset TensorVec 4 [lindex $Tensor 0 0 2]
  lset TensorVec 5 [lindex $Tensor 0 0 1]
  set alpha [__approxFirstToSecondOrderRotation [ __MillerToRotation $miller ]]
  set newTensorVec ""
  for { set i 0 } { $i < 6 } { incr i } {
    lappend newTensorVec ""
    for { set j 0 } { $j < 6 } { incr j } {
      if {[lindex $alpha $i $j] == "-0.0"  || [lindex [split [lindex $TensorVec $j] "*"] 0] == "(-0.0)"} { continue; }
      if {[lindex $alpha $i $j] == "0.0"  || [lindex [split [lindex $TensorVec $j] "*"] 0] == "(0.0)"} { continue; }
      if { [lindex $newTensorVec $i] == "" } {
        lset newTensorVec $i "[expr abs([lindex $alpha $i $j])]*([lindex $TensorVec $j])"
      } else {
        lset newTensorVec $i "[lindex $newTensorVec $i]+[expr abs([lindex $alpha $i $j])]*([lindex $TensorVec $j])"
      }
    }
  }
  #puts $newTensorVec
  #gets stdin
  for { set i 0 } { $i < 6 } { incr i } {
    if {[lindex $newTensorVec $i] == ""} {
      lset newTensorVec $i "0.0"
    }
  }
  set newTensor {{{{} 0 0} {0 {} 0} {0 0 {}}}}
  lset newTensor 0 0 0 [lindex $newTensorVec 0]
  lset newTensor 0 1 1 [lindex $newTensorVec 1]
  lset newTensor 0 2 2 [lindex $newTensorVec 2]
  return $newTensor
}


proc 3DTensorDyadicMult args {
  set name [lindex [info level 0] 0]
  lappend usage "$name TensorList"
  set returns "TensorList"
  set description "Multiples a tensor list together termwise"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set List [lindex $args 0]
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 0 1 "([lindex $List $i 0 1])"
      lset Ten 0 0 2 "([lindex $List $i 0 2])"
      lset Ten 0 1 0 "([lindex $List $i 1 0])"
      lset Ten 0 1 1 "([lindex $List $i 1 1])"
      lset Ten 0 1 2 "([lindex $List $i 1 2])"
      lset Ten 0 2 0 "([lindex $List $i 2 0])"
      lset Ten 0 2 1 "([lindex $List $i 2 1])"
      lset Ten 0 2 2 "([lindex $List $i 2 2])"
    } else {
      lset Ten 0 0 0 "(([lindex $Ten 0 0 0])*([lindex $List $i 0 0]))"
      lset Ten 0 0 1 "(([lindex $Ten 0 0 1])*([lindex $List $i 0 1]))"
      lset Ten 0 0 2 "(([lindex $Ten 0 0 2])*([lindex $List $i 0 2]))"
      lset Ten 0 1 0 "(([lindex $Ten 0 1 0])*([lindex $List $i 1 0]))"
      lset Ten 0 1 1 "(([lindex $Ten 0 1 1])*([lindex $List $i 1 1]))"
      lset Ten 0 1 2 "(([lindex $Ten 0 1 2])*([lindex $List $i 1 2]))"
      lset Ten 0 2 0 "(([lindex $Ten 0 2 0])*([lindex $List $i 2 0]))"
      lset Ten 0 2 1 "(([lindex $Ten 0 2 1])*([lindex $List $i 2 1]))"
      lset Ten 0 2 2 "(([lindex $Ten 0 2 2])*([lindex $List $i 2 2]))"
    }
  }
  
  return $Ten
}

proc 3DTensorAdd args {
  set name [lindex [info level 0] 0]
  lappend usage "$name TensorList"
  set returns "TensorList"
  set description ""
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  set List [lindex $args 0]
  
  set Ten {{{{} {} {}} {{} {} {}} {{} {} {}}}}
  
  for {set i 0} {$i < [llength $List]} {incr i} {
    if { $i == 0 } {
      lset Ten 0 0 0 "([lindex $List $i 0 0])"
      lset Ten 0 0 1 "([lindex $List $i 0 1])"
      lset Ten 0 0 2 "([lindex $List $i 0 2])"
      lset Ten 0 1 0 "([lindex $List $i 1 0])"
      lset Ten 0 1 1 "([lindex $List $i 1 1])"
      lset Ten 0 1 2 "([lindex $List $i 1 2])"
      lset Ten 0 2 0 "([lindex $List $i 2 0])"
      lset Ten 0 2 1 "([lindex $List $i 2 1])"
      lset Ten 0 2 2 "([lindex $List $i 2 2])"
    } else {
      lset Ten 0 0 0 "[lindex $Ten 0 0 0]+([lindex $List $i 0 0])"
      lset Ten 0 0 1 "[lindex $Ten 0 0 1]+([lindex $List $i 0 1])"
      lset Ten 0 0 2 "[lindex $Ten 0 0 2]+([lindex $List $i 0 2])"
      lset Ten 0 1 0 "[lindex $Ten 0 1 0]+([lindex $List $i 1 0])"
      lset Ten 0 1 1 "[lindex $Ten 0 1 1]+([lindex $List $i 1 1])"
      lset Ten 0 1 2 "[lindex $Ten 0 1 2]+([lindex $List $i 1 2])"
      lset Ten 0 2 0 "[lindex $Ten 0 2 0]+([lindex $List $i 2 0])"
      lset Ten 0 2 1 "[lindex $Ten 0 2 1]+([lindex $List $i 2 1])"
      lset Ten 0 2 2 "[lindex $Ten 0 2 2]+([lindex $List $i 2 2])"
    }
  }
  
  return $Ten
}


# Add vectors
proc AddStressVector6 args {
  set name [lindex [info level 0] 0]
  lappend usage "$name VoigtStressVector VoigtStressVector"
  set returns "VoigtStressVector"
  set description "Adds two voigt notation stress vectors together"
  if { [__Help $args $name $usage $returns $description] } {
    return ""
  }
  lassign $args Vec1 Vec2
  set ret ""
  lappend ret [expr [lindex $Vec1 0]+[lindex $Vec2 0]]
  lappend ret [expr [lindex $Vec1 1]+[lindex $Vec2 1]]
  lappend ret [expr [lindex $Vec1 2]+[lindex $Vec2 2]]
  lappend ret [expr [lindex $Vec1 3]+[lindex $Vec2 3]]
  lappend ret [expr [lindex $Vec1 4]+[lindex $Vec2 4]]
  lappend ret [expr [lindex $Vec1 5]+[lindex $Vec2 5]]
  return $ret
}









